#
# Copyright (C) 2025 Indiscale GmbH <info@indiscale.com>
# Copyright (C) 2025 Florian Spreckelsen <f.spreckelsen@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
import linkahead as db

from .utils import get_entity_name


def get_person_identifier(form_data: dict) -> str:
    """Retrieve a person record from the given id, and replace by
    abbreviation if present, else "last name, first name".

    """
    person_rec = db.cached.cached_get_entity_by(eid=form_data["responsible_person"])

    return get_person_identifier_from_rec(person_rec)


def get_person_identifier_from_rec(person_rec: db.Record) -> str:
    # Use abbreviation if present
    if (person_rec.get_property(get_entity_name("abbreviation_prop")) is not None and
            person_rec.get_property(get_entity_name("abbreviation_prop")).value):
        return person_rec.get_property(get_entity_name("abbreviation_prop")).value
    first_name = person_rec.get_property(get_entity_name("first_name_prop")).value if person_rec.get_property(
        get_entity_name("first_name_prop")) is not None else ""
    last_name = person_rec.get_property(get_entity_name("last_name_prop")).value if person_rec.get_property(
        get_entity_name("last_name_prop")) is not None else ""
    # Else use first and last name ...
    if first_name and last_name:
        return f"{last_name}, {first_name}"
    # ... or just last name if no first name
    if last_name:
        return last_name
    # Last fallback id
    return person_rec.id
